package com.tzld.piaoquan.wecom.job;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.common.constant.TimeConstant;
import com.tzld.piaoquan.wecom.common.enums.MessageAttachmentTypeEnum;
import com.tzld.piaoquan.wecom.dao.mapper.HistoryMessageMapper;
import com.tzld.piaoquan.wecom.dao.mapper.StaffWithUserMapper;
import com.tzld.piaoquan.wecom.dao.mapper.UserMapper;
import com.tzld.piaoquan.wecom.model.bo.MiniprogramRecord;
import com.tzld.piaoquan.wecom.model.bo.StaffWithUserParam;
import com.tzld.piaoquan.wecom.model.po.*;
import com.tzld.piaoquan.wecom.service.AccessTokenService;
import com.tzld.piaoquan.wecom.service.HistoryMessageService;
import com.tzld.piaoquan.wecom.service.MessageAttachmentService;
import com.tzld.piaoquan.wecom.utils.HttpClientUtil;
import com.tzld.piaoquan.wecom.utils.HttpPoolClient;
import com.tzld.piaoquan.wecom.utils.MessageUtil;
import com.tzld.piaoquan.wecom.utils.page.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.GET_HISTORY_DATA_URL;

@Slf4j
@Component
public class WeComHistoryDataJob {

    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(30000, 30000, 2000, 5000, 5, 30000);

    final int size = 20;

    @Autowired
    private AccessTokenService accessTokenService;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private HistoryMessageMapper historyMessageMapper;

    @Autowired
    private HistoryMessageService historyMessageService;

    @Autowired
    private MessageAttachmentService messageAttachmentService;

    @Autowired
    private StaffWithUserMapper staffWithUserMapper;

    @XxlJob("saveHistoryMessageJob")
    public ReturnT<String> selectHistoryMessageByDay(String param) {
        StaffWithUserParam staffWithUserParam = new StaffWithUserParam();
        if (StringUtils.isNotEmpty(param)) {
            staffWithUserParam = JSONObject.parseObject(param, StaffWithUserParam.class);
        }
        if (staffWithUserParam.getStartTime() == null) {
            HistoryMessageExample example = new HistoryMessageExample();
            example.setOrderByClause("create_time desc");
            example.setPage(new Page<>(1, 1));
            List<HistoryMessage> historyMessages = historyMessageMapper.selectByExample(example);
            staffWithUserParam.setStartTime(historyMessages.get(0).getCreateTime().getTime() / 1000);
        }
        if (staffWithUserParam.getEndTime() == null) {
            staffWithUserParam.setEndTime(System.currentTimeMillis() / 1000);
        }
        Long startTime = staffWithUserParam.getStartTime();
        Long endTime = staffWithUserParam.getEndTime();

        for (; startTime < endTime; startTime += TimeConstant.DAY) {
            selectHistoryMessage(startTime, Math.min(startTime + TimeConstant.DAY, endTime), staffWithUserParam.getStaffId());
        }
        return ReturnT.SUCCESS;
    }

    public void selectHistoryMessage(Long startTime, Long endTime, Long staffId) {
        try {
            Integer total = getHistoryDataTotal(startTime, endTime);
            if (total == null || total == 0) {
                return;
            }
            int page = total / size + 1;
            for (int n = 0; n < page; n++) {
                String res = getHistoryData(size, n * size, startTime, endTime);
                log.info("selectHistoryMessage size={}, n={}, startTime={}, endTime={}, res={}", size, n, startTime, endTime, res);
                if (ObjectUtils.isEmpty(res)) {
                    return;
                }
                JSONObject jsonObject = JSONObject.parseObject(res);
                JSONArray jsonArray = jsonObject.getJSONArray("missions");
                for (int i = 0; i < jsonArray.size(); i++) {
                    List<String> allExternalUserList = new ArrayList<>();
                    List<MiniprogramRecord> miniprogramRecordList = new ArrayList<>();
                    List<MessageAttachment> messageAttachmentList = new ArrayList<>();
                    Date createdAt = jsonArray.getJSONObject(i).getDate("created_at");
                    JSONArray attachments = jsonArray.getJSONObject(i).getJSONObject("msg_data").getJSONArray("attachments");
                    for (int j = 0; j < attachments.size(); j++) {
                        MiniprogramRecord miniprogramRecord = new MiniprogramRecord();
                        MessageAttachment messageAttachment = new MessageAttachment();
                        String indexPage = attachments.getJSONObject(j).getJSONObject("miniprogram").getString("page");
                        Long videoId = MessageUtil.getVideoId(indexPage);
                        miniprogramRecord.setVideoId(videoId);
                        miniprogramRecord.setAttachmentIdx(j + 1);
                        miniprogramRecordList.add(miniprogramRecord);

                        String cover = attachments.getJSONObject(j).getJSONObject("miniprogram").getString("cover");
                        String appid = attachments.getJSONObject(j).getJSONObject("miniprogram").getString("appid");
                        String title = attachments.getJSONObject(j).getJSONObject("miniprogram").getString("title");
                        messageAttachment.setAppid(appid);
                        messageAttachment.setCover(cover);
                        messageAttachment.setPage(indexPage);
                        messageAttachment.setTitle(title);
                        messageAttachment.setMiniprogramVideoId(videoId);
                        messageAttachment.setSendTime(createdAt);
                        messageAttachment.setType(MessageAttachmentTypeEnum.MINIPROGRAM.getType());
                        messageAttachmentList.add(messageAttachment);
                    }
                    JSONArray subMissionList = jsonArray.getJSONObject(i).getJSONArray("sub_mission_list");
                    for (int k = 0; k < subMissionList.size(); k++) {
                        if (!"sended".equals(subMissionList.getJSONObject(k).getString("status"))) {
                            continue;
                        }
                        List<String> externalUserList = subMissionList.getJSONObject(k).getJSONArray("external_user_list").toJavaList(String.class);
                        if (CollectionUtils.isEmpty(externalUserList)) {
                            continue;
                        }
                        allExternalUserList.addAll(externalUserList);
                    }
                    insertHistoryMessageList(allExternalUserList, miniprogramRecordList, createdAt, staffId);
                    messageAttachmentService.addMiniprogram(messageAttachmentList);
                }
            }
        } catch (IOException e) {
            log.error("selectHistoryMessage error", e);
        }
    }


    private void insertHistoryMessageList(List<String> allExternalUserList, List<MiniprogramRecord> miniprogramRecordList, Date sendTime, Long staffId) {
        List<HistoryMessage> historyMessageList = new ArrayList<>();
        for (String externalUserId3rdParty : allExternalUserList) {
            Long userId = userMapper.selectIdByExternalUserId3rdParty(externalUserId3rdParty);
            if (userId == null) {
                continue;
            }
            if (staffId != null) {
                StaffWithUserExample example = new StaffWithUserExample();
                example.createCriteria().andStaffIdEqualTo(staffId).andUserIdEqualTo(userId);
                List<StaffWithUser> staffWithUserList = staffWithUserMapper.selectByExample(example);
                if (staffWithUserList == null) {
                    continue;
                }
            }
            for (MiniprogramRecord miniprogramRecord : miniprogramRecordList) {
                HistoryMessage historyMessage = new HistoryMessage();
                historyMessage.setSendTime(sendTime);
                historyMessage.setAttachmentIdx(miniprogramRecord.getAttachmentIdx());
                historyMessage.setVideoId(miniprogramRecord.getVideoId());
                historyMessage.setUserId(userId);
                historyMessageList.add(historyMessage);
            }
        }
        historyMessageService.batchInsertHistoryMessage(historyMessageList);
    }

    public Integer getHistoryDataTotal(Long startTime, Long endTime) throws IOException {
        String res = getHistoryData(1, 0, startTime, endTime);
        JSONObject jsonObject = JSONObject.parseObject(res);
        return jsonObject.getInteger("total_count");
    }

    public String getHistoryData(Integer limit, Integer offset, Long startTime, Long endTime) throws IOException {
        String accessToken = accessTokenService.getAccessToken();
        String url = GET_HISTORY_DATA_URL
                + "?access_token=" + accessToken
                + "&limit=" + limit + "&offset=" + offset + "&start_time=" + startTime + "&end_time=" + endTime;
        return httpPoolClientDefault.get(url);
    }
}






