package com.tzld.piaoquan.wecom.job;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.odps.data.Record;
import com.google.common.collect.Lists;
import com.tzld.piaoquan.wecom.dao.mapper.*;
import com.tzld.piaoquan.wecom.model.bo.PushMessage;
import com.tzld.piaoquan.wecom.model.po.*;
import com.tzld.piaoquan.wecom.service.MessageAttachmentService;
import com.tzld.piaoquan.wecom.service.MessageService;
import com.tzld.piaoquan.wecom.utils.DateUtil;
import com.tzld.piaoquan.wecom.utils.MessageUtil;
import com.tzld.piaoquan.wecom.utils.OdpsUtil;
import com.tzld.piaoquan.wecom.utils.page.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.GUARANTEED_MINIPROGRAM_KEY;
import static com.tzld.piaoquan.wecom.common.constant.TimeConstant.MILLISECOND_DAY;

@Log4j2
@Component
public class WeComMessageDataJob {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private MessageService messageService;

    @Autowired
    private MessageAttachmentService messageAttachmentService;

    @Autowired
    private StaffWithUserMapper staffWithUserMapper;

    @Autowired
    private StaffMapper staffMapper;

    @Autowired
    SendMessageMapper sendMessageMapper;

    private static final int MAX_VIDEO_NUM = 3;

    //历史优质视频可推送用户列表
    List<PushMessage> goodHistoryPushList = new ArrayList<>();

    //保底视频列表
    List<Long> guaranteedVideoIdList = new ArrayList<>();

    //从缓存中获取的保底视频数量
    int getGuaranteedVideoIdNum = 0;

    Map<String, String> pageMap = new HashMap<>();

    //初始化操作
    void init() {
        //历史优质视频获取
        String sql = String.format("SELECT * FROM loghubods.history_good_video_can_push_user_list where dt = %s;", DateUtil.getBeforeDayDateString());
        List<Record> recordList = OdpsUtil.getOdpsData(sql);
        if (CollectionUtils.isEmpty(recordList)) {
            return;
        }
        List<PushMessage> list = new ArrayList<>();
        for (Record record : recordList) {
            PushMessage pushMessage = new PushMessage();
            Long videoId = Long.parseLong((String) record.get(0));
            Set<Long> userIds = new HashSet<>(JSONObject.parseArray((String) record.get(1), Long.class));
            pushMessage.setVideoId(videoId);
            pushMessage.setUserIds(userIds);
            list.add(pushMessage);
        }
        goodHistoryPushList = list;

        getGuaranteedVideoIdNum = 0;
        //保底视频获取
        List<Long> videoIdList = Objects.requireNonNull(redisTemplate.opsForList().range(GUARANTEED_MINIPROGRAM_KEY, 0, -1))
                .stream().map(o -> (Integer) o).map(String::valueOf).map(Long::parseLong).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(videoIdList)) {
            log.error("推送消息初始化失败，保底数据为空");
            throw new RuntimeException("保底数据为空");
        }
        List<Long> saveVideoIds = new ArrayList<>();
        for (Long videoId : videoIdList) {
            getGuaranteedVideoIdNum++;
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria().andMiniprogramVideoIdEqualTo(videoId);
            List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachmentList)) {
                continue;
            }
            MessageAttachment messageAttachment = messageAttachmentList.get(0);
            if (messageAttachment.getSendTime() != null
                    && DateUtil.dateDifference(new Date(), messageAttachment.getSendTime()) < 180 * MILLISECOND_DAY) {
                continue;
            }
            saveVideoIds.add(videoId);
            if (saveVideoIds.size() >= MAX_VIDEO_NUM) {
                break;
            }
        }
        if (saveVideoIds.size() < MAX_VIDEO_NUM) {
            log.error("推送消息初始化失败，保底数据不足");
            throw new RuntimeException("保底数据不足");
        }
        guaranteedVideoIdList = saveVideoIds;
    }

    @XxlJob("assembleSendMessageJob")
    public ReturnT<String> assembleSendMessage(String param) {
        init();
        Long staffId = null;
        if (StringUtils.isNotEmpty(param)) {
            staffId = Long.parseLong(param);
        }
        UserExample example = new UserExample();
        long count = userMapper.countByExample(example);
        int page = 1;
        int pageSize = 1000;
        long totalPageSize = count / pageSize + 1;
        for (; page <= totalPageSize; page++) {
            example.setPage(new Page<>(page, pageSize));
            List<User> userList = userMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(userList)) {
                continue;
            }
            //落库逻辑
            List<SendMessage> allSeneMessageList = new ArrayList<>();
            for (User user : userList) {
                List<SendMessage> sendMessageList = getSendMessage(user, staffId);
                if (!CollectionUtils.isEmpty(sendMessageList)) {
                    allSeneMessageList.addAll(sendMessageList);
                }
            }
            if (!CollectionUtils.isEmpty(allSeneMessageList)) {
                sendMessageMapper.insertList(allSeneMessageList);
            }
        }
        //组装好当天要发送的消息后  记录时间  删除保底数据
        saveGuaranteedVideoIdList(guaranteedVideoIdList);
        return ReturnT.SUCCESS;
    }

    public void saveGuaranteedVideoIdList(List<Long> videoIdList) {
        MessageAttachmentExample example = new MessageAttachmentExample();
        example.createCriteria().andMiniprogramVideoIdIn(videoIdList);
        List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            MessageAttachment updateMessageAttachment = new MessageAttachment();
            updateMessageAttachment.setId(messageAttachment.getId());
            updateMessageAttachment.setSendTime(new Date());
            messageAttachmentMapper.updateByPrimaryKeySelective(updateMessageAttachment);
        }
        log.info("getGuaranteedVideoIdNum={}", getGuaranteedVideoIdNum);
        //移除从redis中获取的保底数据
        for (int i = 0; i < getGuaranteedVideoIdNum; i++) {
            redisTemplate.opsForList().leftPop(GUARANTEED_MINIPROGRAM_KEY);
        }
    }

    public List<SendMessage> getSendMessage(User user, Long staffId) {
        StaffWithUserExample example = new StaffWithUserExample();
        StaffWithUserExample.Criteria criteria = example.createCriteria();
        criteria.andUserIdEqualTo(user.getId());
        if (staffId != null) {
            criteria.andUserIdEqualTo(staffId);
        }
        List<StaffWithUser> staffWithUserList = staffWithUserMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(staffWithUserList)) {
            return null;
        }
        int n = 0;
        List<SendMessage> sendMessageList = new ArrayList<>();
        SendMessage sendMessage = new SendMessage();
        for (PushMessage pushMessage : goodHistoryPushList) {
            if (pushMessage.getUserIds().contains(user.getId())) {
                if (n == 0) {
                    sendMessage.setVideoId1(pushMessage.getVideoId());
                }
                if (n == 1) {
                    sendMessage.setVideoId2(pushMessage.getVideoId());
                }
                if (n == 2) {
                    sendMessage.setVideoId3(pushMessage.getVideoId());
                }
                n++;
                if (n >= MAX_VIDEO_NUM) {
                    break;
                }
            }
        }
        //保底数据
        if (n < MAX_VIDEO_NUM) {
            for (Long videoId : guaranteedVideoIdList) {
                if (n == 0) {
                    sendMessage.setVideoId1(videoId);
                }
                if (n == 1) {
                    sendMessage.setVideoId2(videoId);
                }
                if (n == 2) {
                    sendMessage.setVideoId3(videoId);
                }
                n++;
                if (n >= MAX_VIDEO_NUM) {
                    break;
                }
            }
        }
        if (n < MAX_VIDEO_NUM) {
            log.error("组装数据失败 user={}", user);
            return null;
        }
        for (StaffWithUser staffWithUser : staffWithUserList) {
            SendMessage newSendMessage = new SendMessage();
            BeanUtils.copyProperties(sendMessage, newSendMessage);
            newSendMessage.setStaffId(staffWithUser.getStaffId());
            newSendMessage.setUserId(staffWithUser.getUserId());
            sendMessageList.add(newSendMessage);
        }
        return sendMessageList;
    }


    @XxlJob("pushSendMessageJob")
    public ReturnT<String> pushSendMessage(String param) {
        List<SendMessage> groupList = sendMessageMapper.getGroupList(DateUtil.getThatDayDate(), 0);
        if (StringUtils.isNotEmpty(param)) {
            groupList = groupList.stream().filter(e -> e.getStaffId() == Long.parseLong(param)).collect(Collectors.toList());
        }
        if (CollectionUtils.isEmpty(groupList)) {
            return ReturnT.SUCCESS;
        }
        for (SendMessage sendMessage : groupList) {
            sendMessage.setIsSend(0);
            sendMessage.setCreateTime(DateUtil.getThatDayDate());
            List<String> sendUserList = sendMessageMapper.selectExternalUserId3rdParty(sendMessage);
            boolean flag = pushMessage(sendUserList, sendMessage);
            if (flag) {
                SendMessage updateSendMessage = new SendMessage();
                updateSendMessage.setIsSend(1);
                SendMessageExample example = new SendMessageExample();
                example.createCriteria()
                        .andVideoId1EqualTo(sendMessage.getVideoId1())
                        .andVideoId2EqualTo(sendMessage.getVideoId2())
                        .andVideoId3EqualTo(sendMessage.getVideoId3())
                        .andStaffIdEqualTo(sendMessage.getStaffId())
                        .andCreateTimeGreaterThan(DateUtil.getThatDayDate());
                sendMessageMapper.updateByExampleSelective(updateSendMessage, example);
            }

        }
        return ReturnT.SUCCESS;
    }

    public boolean pushMessage(List<String> sendUserList, SendMessage sendMessage) {
        List<JSONObject> pushList = new ArrayList<>();
        StaffExample staffExample = new StaffExample();
        staffExample.createCriteria().andIdEqualTo(sendMessage.getStaffId());
        List<Staff> staffList = staffMapper.selectByExample(staffExample);
        Staff staff = staffList.get(0);
        String text = messageService.getMessageText();
        String name = MessageUtil.getName();

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("name", name);
        jsonObject.put("text", text);
        JSONArray attachments = new JSONArray();
        List<Long> videoIdList = new ArrayList<>();
        videoIdList.add(sendMessage.getVideoId1());
        videoIdList.add(sendMessage.getVideoId2());
        videoIdList.add(sendMessage.getVideoId3());
        for (Long videoId : videoIdList) {
            JSONObject attachment = new JSONObject();
            attachment.put("msgtype", "miniprogram");
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria().andMiniprogramVideoIdEqualTo(videoId);
            List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachmentList)) {
                throw new RuntimeException("附件信息查询异常");
            }
            MessageAttachment messageAttachment = messageAttachmentList.get(0);
            JSONObject miniprogram = new JSONObject();
            miniprogram.put("appid", messageAttachment.getAppid());
            miniprogram.put("title", messageAttachment.getTitle());
            miniprogram.put("cover", messageAttachment.getCover());
            String page = "";
            String key = staff.getStaffExtId() + "_" + videoId;
            if (pageMap.containsKey(key)) {
                page = pageMap.get(key);
            } else {
                page = messageAttachmentService.getPage(staff, videoId);
                pageMap.put(key, page);
            }
            if (StringUtils.isEmpty(page)) {
                throw new RuntimeException("获取page失败");
            }
            miniprogram.put("page", page);

            attachment.put("miniprogram", miniprogram);
            attachments.add(attachment);
        }
        jsonObject.put("attachments", attachments);
        List<List<String>> lists = Lists.partition(sendUserList, 10000);
        for (List<String> list : lists) {
            List<JSONObject> staffEuList = new ArrayList<>();
            JSONObject newJSONObject = new JSONObject();
            newJSONObject.putAll(jsonObject);
            JSONObject staff_eu = new JSONObject();
            staff_eu.put("staff_ext_id", staff.getStaffExtId());
            staff_eu.put("eu_ext_ids", list);
            staffEuList.add(staff_eu);
            newJSONObject.put("staff_eu_list", staffEuList);
            pushList.add(newJSONObject);
        }
        if (CollectionUtils.isEmpty(pushList)) {
            return false;
        }
        for (JSONObject pushJsonObject : pushList) {
            log.info("pushMessage pushJsonObject={}", pushJsonObject);
            boolean flag = messageService.pushMessage(pushJsonObject);
            if (!flag) {
                return flag;
            }
        }
        return true;
    }
}
