package com.tzld.piaoquan.wecom.service.Impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.common.enums.MessageAttachmentTypeEnum;
import com.tzld.piaoquan.wecom.common.exception.CustomizeException;
import com.tzld.piaoquan.wecom.dao.mapper.MessageAttachmentMapper;
import com.tzld.piaoquan.wecom.model.bo.AdPutFlowParam;
import com.tzld.piaoquan.wecom.model.bo.VideoDetail;
import com.tzld.piaoquan.wecom.model.po.MessageAttachment;
import com.tzld.piaoquan.wecom.model.po.MessageAttachmentExample;
import com.tzld.piaoquan.wecom.model.po.Staff;
import com.tzld.piaoquan.wecom.service.MessageAttachmentService;
import com.tzld.piaoquan.wecom.utils.DateUtil;
import com.tzld.piaoquan.wecom.utils.HttpClientUtil;
import com.tzld.piaoquan.wecom.utils.HttpPoolClient;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.*;

import static com.tzld.piaoquan.wecom.common.constant.MessageConstant.appid;
import static com.tzld.piaoquan.wecom.common.constant.OtherServerURL.POST_ADD_TENCENT;
import static com.tzld.piaoquan.wecom.common.constant.OtherServerURL.POST_VIDEO_DETAIL_URL;
import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.GUARANTEED_MINIPROGRAM_KEY;
import static com.tzld.piaoquan.wecom.common.enums.ExceptionCodeEnum.PARAMS_ERROR;


@Service
public class MessageAttachmentServiceImpl implements MessageAttachmentService {

    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(30000, 30000, 2000, 5000, 5, 30000);

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Override
    public void addMiniprogram(List<MessageAttachment> messageAttachmentList) {
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria()
                    .andAppidEqualTo(messageAttachment.getAppid())
                    .andTypeEqualTo(messageAttachment.getType())
                    .andMiniprogramVideoIdEqualTo(messageAttachment.getMiniprogramVideoId());
            List<MessageAttachment> messageAttachments = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachments)) {
                messageAttachmentMapper.insert(messageAttachment);
                continue;
            }
            messageAttachment.setId(messageAttachments.get(0).getId());
            messageAttachmentMapper.updateByPrimaryKeySelective(messageAttachment);
        }
    }

    @Override
    public void createGuaranteedMiniprogram(List<Long> videoIds) {
        if (CollectionUtils.isEmpty(videoIds)) {
            throw new CustomizeException(PARAMS_ERROR);
        }
        Map<Long, VideoDetail> coverMap = getVideoDetail(videoIds);
        List<MessageAttachment> messageAttachmentList = new ArrayList<>();
        for (Long videoId : videoIds) {
            MessageAttachment messageAttachment = new MessageAttachment();
            VideoDetail videoDetail = coverMap.get(videoId);
            if (videoDetail == null || StringUtils.isEmpty(videoDetail.getCover()) || StringUtils.isEmpty(videoDetail.getTitle())) {
                throw new RuntimeException("获取视频详情异常");
            }
            messageAttachment.setMiniprogramVideoId(videoId);
            messageAttachment.setType(MessageAttachmentTypeEnum.MINIPROGRAM.getType());
            messageAttachment.setCover(videoDetail.getCover());
            messageAttachment.setTitle(videoDetail.getTitle());
            messageAttachment.setAppid(appid);
            messageAttachmentList.add(messageAttachment);
        }
        addMiniprogram(messageAttachmentList);
        for (Long video : videoIds) {
            redisTemplate.opsForList().rightPush(GUARANTEED_MINIPROGRAM_KEY, video);
        }
    }

    public Map<Long, VideoDetail> getVideoDetail(List<Long> videoIdList) {
        try {
            Map<Long, VideoDetail> map = new HashMap<>();
            JSONObject params = new JSONObject();
            params.put("videoIdList", videoIdList);
            String post = httpPoolClientDefault.post(POST_VIDEO_DETAIL_URL, params.toJSONString());
            JSONObject res = JSONObject.parseObject(post);
            JSONArray data = res.getJSONArray("data");
            for (int i = 0; i < data.size(); i++) {
                VideoDetail videoDetail = new VideoDetail();
                JSONObject jsonObject = data.getJSONObject(i);
                Long videoId = jsonObject.getLong("id");
                String shareImgPath = jsonObject.getString("shareImgPath");
                String title = jsonObject.getString("title");
                videoDetail.setCover(shareImgPath);
                videoDetail.setTitle(title);
                map.put(videoId, videoDetail);
            }
            return map;
        } catch (IOException e) {
            throw new RuntimeException("获取详情异常");
        }
    }

    @Override
    public String getPage(Staff staff, Long videoId) {
        try {
            if (staff == null || staff.getCarrierId() == null) {
                return "";
            }
            String page = selectPage(staff.getCarrierId(), videoId);
            if (StringUtils.isNotEmpty(page)) {
                return page;
            }
            return insertPage(staff.getCarrierId(), videoId, staff.getRemark());
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    private String insertPage(String carrierId, Long videoId, String remark) throws IOException {
        AdPutFlowParam param = new AdPutFlowParam();
        param.setChannel("tencentqw");
        param.setPath("pages/category.html");
        param.setPutCarrierId(carrierId);
        param.setPutScene("touliu");
        param.setPutStartDate(DateUtil.getDayDateString("yyyy-MM-dd"));
        param.setPutTypeOne("企微");
        param.setPutTypeTwo("日常推送");
        param.setPutTypeThree("位置X");
        param.setRemark(remark);
        param.setVideoId(videoId);
        Map<String, String> requestParam = new HashMap<>();
        String jumpPage = "pages/user-videos?id=${videoId}&fromGzh=1&rootShareId=${uuid}&shareId=${uuid}&rootSourceId=[rootSourceId]"
                .replace("${videoId}", "" + videoId)
                .replace("${uuid}", "" + UUID.randomUUID());
        requestParam.put("jumpPage", jumpPage);
        param.setRequestParam(requestParam);
        String res = httpPoolClientDefault.post(POST_ADD_TENCENT, JSONObject.toJSONString(param));
        JSONObject jsonObject = JSONObject.parseObject(res);
        JSONObject data = jsonObject.getJSONObject("data");
        return data.getString("url");
    }

    public String selectPage(String carrierId, Long videoId) throws IOException {
        Map<String, String> map = new HashMap<>();
        String url = "https://api.piaoquantv.com/ad/put/flow/list/tencent?videoId=${videoId}&putScene=touliu&channel=tencentqw&currentPage=1&pageSize=20"
                .replace("${videoId}", "" + videoId);
        String res = httpPoolClientDefault.get(url);
        JSONObject jsonObject = JSONObject.parseObject(res);
        JSONObject data = jsonObject.getJSONObject("data");
        JSONArray objs = data.getJSONArray("objs");
        for (int i = 0; i < objs.size(); i++) {
            JSONObject obj = objs.getJSONObject(i);
            String putCarrierId = obj.getString("putCarrierId");
            String page = obj.getString("url");
            if (StringUtils.isEmpty(putCarrierId) || StringUtils.isEmpty(page)) {
                continue;
            }
            map.put(putCarrierId, page);
        }
        return map.get(carrierId);
    }
}
